table td {
    text-overflow: ellipsis;
    overflow: hidden;
}

table th {
    text-overflow: ellipsis;
    overflow: hidden;
}

thead input {
    width: 100%;
    padding: 3px;
    box-sizing: border-box;
}

tfoot input {
    width: 100%;
    padding: 3px;
    box-sizing: border-box;
}
